#pragma rtGlobals=3		// Use modern global access method and strict wave access.

Menu "HPC"
	"Display", HPC_displayWaves()
	"Display TPD", HPC_displayTPD()
	"Data processing", HPC_DataProcessing()
	 "Generate Layout",HPC_CreateLayout()
end


function HPC_displayWaves()
	String datafolderStr = GetDataFolder(1)
	variable len=strlen(datafolderStr)
	String expNum = datafolderStr[len-6,len-3]
	variable i=0
	variable j=0
	String massStr,timeStr,axisStr,currStr,graphNameStr,tempStr
	graphNameStr="HPC"+datafolderStr[6,len-3]
	// determine how many partial pressure waves to display
	do
		massStr="MASS"+num2str(i)+expNum
		timeStr="TIME"+num2str(i)+expNum
		if (exists(massStr)==1 && exists(timeStr)==1)
			j+=1
		endif
		i+=1
	while(i<100)
	// check if T wave exsists and determine npoints
	variable npnts
	tempStr="celsiusTC"+expNum
	if (exists(tempStr)==1)
		 npnts=numpnts($tempStr)
		j+=1
	endif
	String tctimeStr="TIME"+expNum
	make/n=(npnts)/O $tctimeStr
	variable nwaves=j
	// display all waves
	i=0
	j=0
	variable arange=1/nwaves
	String labelStr
	variable flag=0
	do
		massStr="MASS"+num2str(i)+expNum
		timeStr="TIME"+num2str(i)+expNum
		currStr="HTNCR"+expNum
		if ((exists(massStr)==1) && (exists(timeStr)==1))
			if (flag==0)
				duplicate/O $timeStr $tctimeStr
				flag=1
			endif
			axisStr="PP"+num2str(i)
			SIS_graphToFrontLeft(graphNameStr,massStr,timeStr,axisStr)
			ModifyGraph freePos($axisStr)=0
			ModifyGraph axisEnab($axisStr)={j*arange,(j+1)*arange-0.02}
			ModifyGraph mirror($axisStr)=1,lblPos($axisStr)=90;
			labelStr="\\F'Arial'\\Z14Mass "+num2str(i)+"\r(arb.u.)"
			Label $axisStr labelStr
			if (i==28)
				ModifyGraph rgb($massStr)=(65280,0,0)
			endif
			if (i==32)
				ModifyGraph rgb($massStr)=(0,52224,0)
			endif
			if (i==44)
				ModifyGraph rgb($massStr)=(0,15872,65280)
			endif
			if (i==18)
				ModifyGraph rgb($massStr)=(65280,43520,0)
			endif
			ModifyGraph mode($massStr)=4,marker($massStr)=19,msize($massStr)=1.1
			j+=1
		endif
		i+=1
	while(i<100)
	// append T wave
	if (exists(tempStr)==1)
		axisStr="TempC"
		SIS_graphToFrontLeft(graphNameStr,tempStr,tctimeStr,axisStr)
		ModifyGraph freePos($axisStr)=0
		ModifyGraph axisEnab($axisStr)={j*arange,(j+1)*arange-0.02}
		ModifyGraph mirror($axisStr)=1,lblPos($axisStr)=98;
		labelStr="\\F'Arial'\\Z14Sample temp.\r(C)"
		Label $axisStr labelStr
		ModifyGraph rgb($tempStr)=(0,0,0)
		ModifyGraph mirror($axisStr)=0
	endif
	if (exists(currStr)==1)
		axisStr="curr"
		SIS_graphToFrontRight(graphNameStr,currStr,tctimeStr,axisStr)
		ModifyGraph freePos($axisStr)=0
		ModifyGraph axisEnab($axisStr)={j*arange,(j+1)*arange-0.02}
		ModifyGraph mirror($axisStr)=1,lblPos($axisStr)=98;
		labelStr="\\F'Arial'\\Z14heating current.\r(A)"
		Label $axisStr labelStr
		ModifyGraph rgb($tempStr)=(0,0,0)
		ModifyGraph mirror($axisStr)=0
	endif
	Label bottom "\\F'Arial'\\Z14Time (ms)"
	ModifyGraph width=250,height={Aspect,1.8}
	doUpdate
	ModifyGraph width=0,height=0
	Legend/K/N=text0
end
	
function HPC_dataProcessing()
	String datafolderStr = GetDataFolder(1)
	variable len=strlen(datafolderStr)
	String expNum = datafolderStr[len-6,len-3]
	variable i=0
	variable j=0
	variable k=0
	String massStr,timestr,ptimeStr,tempStr,axis1Str,axis2Str,axis3Str,graphNameStr,graph1NameStr,graph2NameStr,graph3NameStr,tmpGraphNameStr
	
	// determine how many waves to display
	do
		massStr="MASS"+num2str(i)+expNum
		ptimeStr="PTIME"+num2str(i)+expNum
		if (exists(massStr)==1 && exists(ptimeStr)==1)
			j+=1
		endif
		i+=1
	while(i<100)
	// check if T wave exsists and determine npoints
	variable npnts
	tempStr="celsiusTC"+expNum
	if (exists(tempStr)==1)
		 npnts=numpnts($tempStr)
		j+=1
	endif
	variable nwaves=j // number of waves to append to the graph
	variable arange=1/nwaves
	variable arangePT=1/(nwaves-1)
	String labelStr

	// analysis loop
	i=0
	j=0
	k=0
	variable n=0
	variable m=0
	variable nmax=0
	variable np=0
	variable y1,y2,yavg,yarea,bkgarea,t1,t2,avgtemp
	tmpGraphNameStr="TMP"+datafolderStr[6,len-3]
	graphNameStr="HPC"+datafolderStr[6,len-3]
	graph1NameStr="BPA"+datafolderStr[6,len-3]
	graph2NameStr="PPA"+datafolderStr[6,len-3]
	graph3NameStr="TPA"+datafolderStr[6,len-3]
	String pulseStr,bkgStr,eventTimeStr,eventTempStr
	variable respnts=1
	variable flag=0
	variable ndelta
	variable done = 0
	variable fractbkg = 0.5
	
	do
		massStr="MASS"+num2str(i)+expNum
		ptimeStr="PTIME"+num2str(i)+expNum
		timeStr="TIME"+num2str(i)+expNum
		tempStr="celsiusTC"+expNum
		
		if ((exists(massStr)==1) && (exists(timeStr)==1 ))
	       	
	       	// 1) determine numpnts following each pulse
			wave ptw=$ptimeStr
			wave tw=$timeStr
			wave mw=$massStr
			wave tcw=$tempStr
			
			np=numpnts(ptw) // total number of data points
			n=0
			do 
				n+=1
			while ((ptw[n+1]-ptw[n]>0) && ((n+1)<np-1))
			nmax=n+1 // this is the number of data points after each pulse
			ndelta = floor(nmax/10)+1
	       	respnts=floor(np/nmax)-1
			if (done ==0)
	       		print "readout per pulse: ", nmax, ", Total number of pulses: ",respnts,"delta",ndelta
	       		done = 1
	       	endif
	       	
	       	// 2) make and display temp wave
	       	make/N=(nmax)/O massw=NaN,timew=NaN,tempw=NaN,blw=NaN
	       	SIS_graphToFront(tmpGraphNameStr,"massw","timew")
	       	 
	       	// 3) extract subset corresonding to a single pulse
	       	// results waves
			pulseStr="PL"+num2str(i)+datafolderStr[len-6,len-3]
			bkgStr="BK"+num2str(i)+datafolderStr[len-6,len-3]
			eventTimeStr="PT"+num2str(i)+datafolderStr[len-6,len-3]
			eventTempStr="ETEMP"+datafolderStr[len-6,len-3]
			make/N=(respnts)/O $pulseStr,$bkgStr,$eventTimeStr,$eventTempStr
			wave plsw=$pulseStr
			wave bkgw=$bkgStr
			wave etimew=$eventTimeStr
			wave etempw=$eventTempStr
			
	       	// loop
	       	massw=0
	       	for (j=0;j<respnts;j+=1)
	       		massw[]=mw[j*(nmax)+p-ndelta]
				timew[]=tw[j*(nmax)+p-ndelta]
				timew-=tw[j*(nmax)+0*ndelta]
				tempw[]=tcw[j*(nmax)+p-ndelta] //avg temp
	       		// for each mass != 0 we want to determine pavg and peak integral
	   			duplicate/o massw blinew
      				wavestats/Q massw
	        		y1=mean(massw,0,round(fractbkg*ndelta))
       			y2=mean(massw,nmax-1-round(fractbkg*ndelta),nmax-1)
       			t1=timew[0]
       			t2=timew[nmax-1]
	        		blinew[]=y1+(y2-y1)*(p)/(nmax)
	        		massw[]-=y1+(y2-y1)*(p)/(nmax-1) // removes a linear background
	        		// show results
				doUpdate
	        		sleep/S 0.05	        		
       			yarea=areaXY(timew,massw) /(t2-t1)
                   		bkgarea=areaXY(timew,blinew) /(t2-t1)
             		plsw[j]=yarea
             		bkgw[j]=bkgarea //(t2-t1)*yavg //or bkgarea
             		etimew[j]=tw[j*(nmax)]
             		if (j==0)
				  plsw[j]=NaN
			endif
             		if (flag == 0)
             			//CurveFit/NTHR=0/Q line tempw /D 
       				//wave fit_tempw
       				avgtemp=mean(tempw,0,nmax-1)
       				etempw[j]=avgtemp
       			endif
             	endfor
             	
             	// display results
             	axis1Str="PB"+num2str(i)
             	axis2Str="PP"+num2str(i)
             	axis3Str="TP"+num2str(i)
			// background
			SIS_graphToFrontLeft(graph1NameStr,bkgStr,eventTimeStr,axis1Str)
			MoveWindow 400, 0, 0, 0
			ModifyGraph width=250,height={Aspect,1.8}
			doUpdate
			ModifyGraph mirror($axis1Str)=0
			ModifyGraph freePos($axis1Str)=0
			ModifyGraph axisEnab($axis1Str)={k*arange,(k+1)*arange-0.02}
			ModifyGraph lblPos($axis1Str)=90;
			ModifyGraph mirror($axis1Str)=1
			labelStr="\\F'Arial'\\Z14Mass "+num2str(i)+"\r(arb.u.)"
			Label $axis1Str labelStr
			TextBox/C/N=text1/A=LB "Background partial pressures"
			doUpdate
			if (i==28)
				ModifyGraph rgb($bkgStr)=(61000,0,0)
			endif
			if (i==32)
				ModifyGraph rgb($bkgStr)=(0,48000,0)
			endif
			if (i==44)
				ModifyGraph rgb($bkgStr)=(0,12000,65000)
			endif
			if (i==18)
				ModifyGraph rgb($bkgStr)=(65280,43520,0)
			endif	
			ModifyGraph mode($bkgStr)=4,marker($bkgStr)=19,msize($bkgStr)=1.1
			doUpdate				
			// pulse
			SIS_graphToFrontLeft(graph2NameStr,pulseStr,eventTimeStr,axis2Str)
			MoveWindow 800, 0, 0, 0
			ModifyGraph width=250,height={Aspect,1.8}
			doUpdate
			ModifyGraph mirror($axis2Str)=0
			ModifyGraph freePos($axis2Str)=0
			ModifyGraph axisEnab($axis2Str)={k*arange,(k+1)*arange-0.02}
			ModifyGraph lblPos($axis2Str)=90;
			ModifyGraph mirror($axis2Str)=1
			labelStr="\\F'Arial'\\Z14Mass "+num2str(i)+"\r(arb.u.)"
			Label $axis2Str labelStr
			TextBox/C/N=text1/A=LB "Pulse partial pressures"
			doUpdate
			if (i==28)
				ModifyGraph rgb($pulseStr)=(61000,0,0)
			endif
			if (i==32)
				ModifyGraph rgb($pulseStr)=(0,48000,0)
			endif
			if (i==44)
				ModifyGraph rgb($pulseStr)=(0,12000,65000)
			endif
			if (i==18)
				ModifyGraph rgb($pulseStr)=(65280,43520,0)
			endif
			ModifyGraph mode($pulseStr)=4,marker($pulseStr)=19,msize($pulseStr)=1.1
			doUpdate
			// P vs T
			SIS_graphToFrontLeft(graph3NameStr,pulseStr,eventTempStr,axis3Str)
			MoveWindow 1200, 0, 0, 0
			ModifyGraph width=250,height={Aspect,1.8}
			doUpdate
			ModifyGraph mirror($axis3Str)=0
			ModifyGraph freePos($axis3Str)=0
			ModifyGraph axisEnab($axis3Str)={k*arangePT,(k+1)*arangePT-0.02}
			ModifyGraph lblPos($axis3Str)=90;
			ModifyGraph mirror($axis3Str)=1
			labelStr="\\F'Arial'\\Z14Mass "+num2str(i)+"\r(arb.u.)"
			Label $axis3Str labelStr
			TextBox/C/N=text1/A=LB "Pulse partial pressures"
			doUpdate
			if (i==28)
				ModifyGraph rgb($pulseStr)=(61000,0,0)
			endif
			if (i==32)
				ModifyGraph rgb($pulseStr)=(0,48000,0)
			endif
			if (i==44)
				ModifyGraph rgb($pulseStr)=(0,12000,65000)
			endif
			if (i==18)
				ModifyGraph rgb($pulseStr)=(65280,43520,0)
			endif
			ModifyGraph mode($pulseStr)=4,marker($pulseStr)=19,msize($pulseStr)=1.1
			ModifyGraph mode=3,msize=2.5
			doUpdate
             	flag=1
             	k+=1
	      endif
	 	i+=1
	while(i<100)    
	
	DoWindow/K $tmpGraphNameStr
	
	// append T wave
	if (exists(tempStr)==1)
		// background
		axis1Str="TempC"
		SIS_graphToFrontLeft(graph1NameStr,eventTempStr,eventTimeStr,axis1Str)
		ModifyGraph freePos($axis1Str)=0
		ModifyGraph axisEnab($axis1Str)={k*arange,(k+1)*arange-0.02}
		ModifyGraph mirror($axis1Str)=1,lblPos($axis1Str)=98;
		labelStr="\\F'Arial'\\Z14Sample temp.\r(C)"
		Label $axis1Str labelStr
		ModifyGraph rgb($eventTempStr)=(0,0,0)
		Label bottom "\\F'Arial'\\Z14Time (ms)"
		ModifyGraph width=250,height={Aspect,1.8}
		doUpdate
		ModifyGraph width=0,height=0
		Legend/K/N=text0
		// pulse
		axis2Str="TempC"
		SIS_graphToFrontLeft(graph2NameStr,eventTempStr,eventTimeStr,axis2Str)
		ModifyGraph freePos($axis2Str)=0
		ModifyGraph axisEnab($axis2Str)={k*arange,(k+1)*arange-0.02}
		ModifyGraph mirror($axis2Str)=1,lblPos($axis2Str)=98;
		labelStr="\\F'Arial'\\Z14Sample temp.\r(C)"
		Label $axis2Str labelStr
		ModifyGraph rgb($eventTempStr)=(0,0,0)
		Label bottom "\\F'Arial'\\Z14Time (ms)"
		ModifyGraph width=250,height={Aspect,1.8}
		doUpdate
		ModifyGraph width=0,height=0
		Legend/K/N=text0
		// P vs T
		SIS_graphToFrontLeft(graph3NameStr,pulseStr,eventTempStr,axis3Str)
		labelStr="\\F'Arial'\\Z14Sample temp.(C)"
		ModifyGraph rgb($eventTempStr)=(0,0,0)
		Label bottom  labelStr 
		ModifyGraph width=250,height={Aspect,1.8}
		doUpdate
		Legend/K/N=text0
		
		// fix T wave
		Interpolate2/T=2/N=(numpnts($eventTempStr))/E=2/Y=$eventTempStr $tempStr
	endif
	
	HPC_normaliseToMass(44,32)
			
end

function HPC_normaliseToMass(target,reference)
	variable target
	variable reference
	String datafolderStr = GetDataFolder(1)
	variable len=strlen(datafolderStr)
	String expNum = datafolderStr[len-6,len-3]
	String targetpulseStr="PL"+num2str(target)+datafolderStr[len-6,len-3]
	String referencepulseStr="PL"+num2str(reference)+datafolderStr[len-6,len-3]
	String targetbkgStr="BK"+num2str(target)+datafolderStr[len-6,len-3]
	String referencebkgStr="BK"+num2str(reference)+datafolderStr[len-6,len-3]
	wave tplsw=$targetpulseStr
	wave rplsw=$referencepulseStr
	wave tbkgw=$targetbkgStr
	wave rbkgw=$referencebkgStr
	duplicate/o $referencepulseStr tmpw
	CurveFit/NTHR=0 exp $referencepulseStr /D=tmpw 
	tplsw/=tmpw
end


function HPC_CreateLayout()
	String datafolderStr = GetDataFolder(1)
	variable len=strlen(datafolderStr)
	String expNum = datafolderStr[len-6,len-3]
	variable i=0
	variable j=0
	variable k=0
	String graphNameStr,graph1NameStr,graph2NameStr
	graphNameStr="HPC"+datafolderStr[6,len-3]
	graph1NameStr="BPA"+datafolderStr[6,len-3]
	graph2NameStr="PPA"+datafolderStr[6,len-3]
	String layoutStr="HPClayout"+expNum
	NewLayout/P=Landscape as layoutStr
	AppendLayoutobject/F=1 graph $graphNameStr 
	AppendLayoutobject/F=1 graph $graph1NameStr 
	AppendLayoutobject/F=2 graph $graph2NameStr 
	String tileStr="Tile/A=(1,3) "+graphNameStr+","+graph1NameStr+","+graph2NameStr
	Execute tileStr
end
    
function HPC_process()	
// 
String timeStr="PTIME44_010"
String massStr="MASS44_010"
string tempStr="celsiusTC_010"
wave tw=$timeStr
wave mw=$massStr
wave tcw=$tempStr
// determine time length
variable i=0
do 
	i+=1
while (tw[i+1]-tw[i]>0)
variable nmax=i+1
// tmpwaves
make/N=(nmax)/O massw,timew,tempw
display massw vs timew
wave W_coef,fit_massw,fit_tempw
variable j
variable vavg,varea,vtemp
// results waves
variable npoints=floor(numpnts(tw)/i)-1
make/n=(npoints)/O res_T,res_react
variable v1,v2
//for (j=0;j<npoints;j+=1)
for (j=4;j<5;j+=1)
	// extract
	massw[0,(i)]=mw[j*(nmax)+p]
	timew[0,(i)]=tw[j*(nmax)+p]
	tempw[0,(i)]=tcw[j*(nmax)+p] //avg temp
	doUpdate
       // process: 1) determin area
    
       v1=massw[0]
       v2=massw[nmax-1]
      massw[]-=(v1-v2)*(nmax-1-p)/(nmax-1)
      vavg=mean(massw,nmax-nmax/10-1,nmax-1)
      massw-=vavg
      //
       //CurveFit/NTHR=0/Q LogNormal massw /X=timew /D 
//       print j
       sleep/S 0.25
      
       //varea = W_coef[1]//works ok
       varea=areaXY(timew,massw) //works better!
     // varea = area(fit_massw)
       // proecess: 2) determine avg temp
       CurveFit/NTHR=0/Q line  tempw /D 
       vtemp=mean(fit_tempw,0,nmax-1)
//       print varea,vtemp
       res_T[j]=vtemp
       res_react[j]=varea
endfor




end



//---------------------------------------------------
// added 30/07/2016
//---------------------------------------------------
function HPC_displayTPD()
	String datafolderStr = GetDataFolder(1)
	variable len=strlen(datafolderStr)
	String expNum = datafolderStr[len-6,len-3]
	variable i=0
	variable j=0
	String massStr,timeStr,axisStr,currStr,graphNameStr,tempStr
	graphNameStr="TPD"+datafolderStr[6,len-3]
	// determine how many partial pressure waves to display
	do
		massStr="MASS"+num2str(i)+expNum
		timeStr="TIME"+num2str(i)+expNum
		if (exists(massStr)==1 && exists(timeStr)==1)
			j+=1
		endif
		i+=1
	while(i<100)
	// check if T wave exsists and determine npoints
	variable npnts
	tempStr="celsiusTC"+expNum
	if (exists(tempStr)==1)
		print "we are here"
		String tctimeStr="TEMP"+expNum
		make/n=(npnts)/O $tctimeStr
		variable nwaves=j
		// display all waves
		i=0
		j=0
		variable arange=1/nwaves
		String labelStr
		variable flag=0
		do
			massStr="MASS"+num2str(i)+expNum
			timeStr="TIME"+num2str(i)+expNum
			currStr="HTNCR"+expNum
			if ((exists(massStr)==1))
				if (flag==0)
					duplicate/O $tempStr $tctimeStr
					flag=1
				endif
				axisStr="PP"+num2str(i)
				SIS_graphToFrontLeft(graphNameStr,massStr,tempStr,axisStr)
				ModifyGraph freePos($axisStr)=0
				ModifyGraph axisEnab($axisStr)={j*arange,(j+1)*arange-0.02}
				ModifyGraph mirror($axisStr)=1,lblPos($axisStr)=90;
				labelStr="\\F'Arial'\\Z14Mass "+num2str(i)+"\r(arb.u.)"
				Label $axisStr labelStr
				if (i==28)
					ModifyGraph rgb($massStr)=(65280,0,0)
				endif
				if (i==32)
					ModifyGraph rgb($massStr)=(0,52224,0)
				endif
				if (i==44)
					ModifyGraph rgb($massStr)=(0,15872,65280)
				endif
				ModifyGraph mode($massStr)=4,marker($massStr)=19,msize($massStr)=1.1
				j+=1
			endif
			i+=1
		while(i<100)
		// append T wave
		//if (exists(tempStr)==1)
		//	axisStr="TempC"
		//	SIS_graphToFrontLeft(graphNameStr,tempStr,tctimeStr,axisStr)
		//	ModifyGraph freePos($axisStr)=0
		//	ModifyGraph axisEnab($axisStr)={j*arange,(j+1)*arange-0.02}
		//	ModifyGraph mirror($axisStr)=1,lblPos($axisStr)=98;
		//	labelStr="\\F'Arial'\\Z14Sample temp.\r(C)"
		//	Label $axisStr labelStr
		//	ModifyGraph rgb($tempStr)=(0,0,0)
		//	ModifyGraph mirror($axisStr)=0
		//endif
		if (exists(currStr)==1)
			axisStr="curr"
			SIS_graphToFrontRight(graphNameStr,currStr,tctimeStr,axisStr)
			ModifyGraph freePos($axisStr)=0
			ModifyGraph axisEnab($axisStr)={j*arange,(j+1)*arange-0.02}
			ModifyGraph mirror($axisStr)=1,lblPos($axisStr)=98;
			labelStr="\\F'Arial'\\Z14heating current.\r(A)"
			Label $axisStr labelStr
			ModifyGraph mirror($axisStr)=0
		endif
		Label bottom "\\F'Arial'\\Z14Temparature (C)"
		ModifyGraph width=250,height={Aspect,1.8}
		doUpdate
		ModifyGraph width=0,height=0
		Legend/K/N=text0
	
	endif
end